@@ -1,8 +1,9 @@
+#include "iperf_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <sysexits.h>
 #include <stdint.h>
 
 #include <iperf_api.h>
