@@ -27,9 +27,7 @@
 
 
 #include <assert.h>
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#endif
 #include <stdio.h>
 #include <string.h>
 
